@@ -92,14 +92,22 @@ class Pack(object):
         return self.lookup("objects/%s" % fid)
 
     def has_feature(self, fid):
-        fid = self.get_feature_id(fid)
-        ret = self.get(fid)
-        return ret is not None
+        return fid in self.known_ids
 
     def add_feature(self, feature, prev=None):
         desc_id = self.desc_id(feature)
         if prev is not None:
-            feature["previous"] = self.get_feature_id(prev)
+            prev_id = self.get_feature_id(prev)
+            feature["previous"] = prev_id
+            for k, v in self.active_ids.iteritems():
+                if k not in (prev_id, desc_id) and v == prev_id:
+                    raise KeyError(
+                        "Invalid link to previous id: %s -> %s" % (k, v)
+                    )
+            # A feature can be its own description and already be
+            # deleted when removing features.
+            if prev_id in self.active_ids:
+                del self.active_ids[prev_id]
         elif desc_id in self.active_ids:
             raise ValueError("Feature description %r exists." % desc_id)
 
